@@ -1,12 +1,13 @@
 #ifndef __INSTEAD_EXTERNAL_H_
 #define __INSTEAD_EXTERNAL_H_
 
-#ifdef _WIN32_WCE
- #define PATH_MAX 255
+#if defined (_WIN32_WCE) || defined(WINRT)
  #define errno 0
- #define strerror(a) ""
  #define putenv(a) ;
+ #ifndef WINRT
+ #define strerror(a) ""
  #define setlocale(a, b) ;
+ #endif
 #endif
 
 #include <lua.h>
