@@ -927,6 +927,7 @@ void instead_ready(void)
 	}
 	busy_time = 0;
 }
+
 static int luaB_stead_busy(lua_State *L) {
 	int busy = lua_toboolean(L, 1);
 	if (busy) {
@@ -956,6 +957,7 @@ static int luaB_stead_busy(lua_State *L) {
 		return 0;
 	}
 	instead_ready();
+	game_gfx_commit(0);
 	return 0;
 }
 
@@ -2279,7 +2281,7 @@ static int luaB_pixels_sprite(lua_State *L) {
 static int pixels_scale(lua_State *L) {
 	img_t img, img2;
 	int rc;
-	struct lua_pixels *src = (struct lua_pixels*)lua_touserdata(L, 1);;
+	struct lua_pixels *src = (struct lua_pixels*)lua_touserdata(L, 1);
 	float xs = luaL_optnumber(L, 2, 0);
 	float ys = luaL_optnumber(L, 3, 0);
 	int smooth = lua_toboolean(L, 4);
@@ -2301,7 +2303,7 @@ static int pixels_scale(lua_State *L) {
 static int pixels_rotate(lua_State *L) {
 	img_t img, img2;
 	int rc;
-	struct lua_pixels *src = (struct lua_pixels*)lua_touserdata(L, 1);;
+	struct lua_pixels *src = (struct lua_pixels*)lua_touserdata(L, 1);
 	float angle = luaL_optnumber(L, 2, 0);
 	int smooth = lua_toboolean(L, 3);
 
@@ -2499,6 +2501,14 @@ static int luaB_after_callback(lua_State *L) {
 	return 1;
 }
 
+static int luaB_screen_size(lua_State *L) {
+	int w = 0; int h = 0;
+	gfx_get_max_mode(&w, &h, MODE_ANY);
+	lua_pushinteger(L, w);
+	lua_pushinteger(L, h);
+	return 2;
+}
+
 static const luaL_Reg sprites_funcs[] = {
 	{"instead_font_load", luaB_load_font},
 	{"instead_font_free", luaB_free_font},
@@ -2534,6 +2544,7 @@ static const luaL_Reg sprites_funcs[] = {
 	{"instead_noise2", luaB_noise2},
 	{"instead_noise3", luaB_noise3},
 	{"instead_noise4", luaB_noise4},
+	{"instead_screen_size", luaB_screen_size},
 	{"instead_render_callback", luaB_after_callback},
 	{NULL, NULL}
 };
