@@ -5,6 +5,10 @@ local instead = std.obj { nam = '@instead' }
 instead.nosave = false
 instead.noautosave = false
 
+function instead.mouse_filter()
+	return 0
+end
+
 function instead.render_callback()
 	return false
 end
@@ -54,6 +58,10 @@ end
 function instead.restart(v)
 end
 
+function instead.text_input()
+	return false
+end
+
 function instead.atleast(...)
 	return true
 end
@@ -65,6 +73,16 @@ function iface:title(str) -- hide title
 	return str
 end
 
+function iface:img() return end
+
+function iface:imgl() return end
+
+function iface:imgr() return end
+
+function iface:nb(t)
+	return t == '' and ' ' or t
+end
+
 std.stat = std.class({
 	__stat_type = true;
 }, std.obj);
@@ -312,9 +330,16 @@ std.mod_init(function()
 end)
 std.mod_start(function()
 	dict = {}
+	local mp = std.ref '@metaparser'
+	if mp then
+		mp.winsize = 0
+		mp.prompt = false
+	end
 end)
 std.mod_step(function(state)
 	if state then
 		dict = {}
 	end
 end)
+
+require "ext/paths"
