@@ -1,5 +1,5 @@
 stead = {
-	version = "3.2.1",
+	version = "3.3.2",
 	api_version = "1.1.6", -- last version before 1.2.0
 	table = table,
 	delim = ',',
@@ -2134,7 +2134,7 @@ stead.game_save = function(self, name, file)
 	if name == nil then
 		return nil, false
 	end
-	h = stead.io.open(name,"wb");
+	h = stead.io.open(name .. '.tmp', "wb");
 	if not h then
 		return nil, false
 	end
@@ -2148,6 +2148,8 @@ stead.game_save = function(self, name, file)
 	stead.do_savegame(self, h);
 	h:flush();
 	h:close();
+	stead.os.remove(name);
+	stead.os.rename(name .. '.tmp', name);
 	game.autosave = false; -- we have only one try for autosave
 	stead.restart_game = false
 	return nil;
@@ -2186,7 +2188,7 @@ end
 
 game = game {
 	codepage = "UTF-8";
-	nam = [[INSTEAD -- Simple Text Adventure interpreter v]]..stead.version..[[ '2009-2016 by Peter Kosyh]];
+	nam = [[INSTEAD -- Simple Text Adventure interpreter v]]..stead.version..[[ '2009-2020 by Peter Kosyh]];
 	dsc = [[
 Commands:^
     look(or just enter), act <on what> (or just what), use <what> [on what], go <where>,^
@@ -3267,6 +3269,16 @@ local build_sandbox_output = function(realpath, error, type, find, gsub, savepat
 	end)
 end
 
+local build_sandbox_load = function(eval, error, type, find)
+	return stead.hook(eval, function(f, str, ...)
+		if type(str) == 'string' and find(str, "\x1b", 1, true) == 1 then
+			error ("Loading bytecode is forbidden!", 3)
+			return false
+		end
+		return f(str, ...)
+	end)
+end
+
 io.open = build_sandbox_open(instead_realpath, error, type, string.find, string.gsub,
 		instead_savepath(), instead_gamepath());
 
@@ -3294,6 +3306,13 @@ end
 if not DEBUG then
 	debug = nil
 end
+if _VERSION == "Lua 5.1" then
+	loadstring = build_sandbox_load(loadstring, error, type, string.find)
+	stead.eval = loadstring
+else
+	load = build_sandbox_load(load, error, type, string.find)
+	stead.eval = load
+end
 package.cpath = ""
 package.preload = {}
 package = nil
