@@ -37,10 +37,12 @@ local instead = std.ref '@instead'
 -- luacheck: read globals instead_direct
 -- luacheck: read globals instead_busy
 -- luacheck: read globals instead_sprite_pixels
+-- luacheck: read globals instead_screen_size
 
 -- theme
 instead.theme_var = instead_theme_var
 instead.theme_name = instead_theme_name
+instead.screen_size = instead_screen_size
 
 local theme = std.obj {
 	nam = '@theme';
